﻿(* Raima.Rdm is the system module that contains the ADO.NET data provider
   for RDM *)
open Raima.Rdm

(* EXAMPLE - SQL07 for F# using ADO.NET *)
(* This example shows how to use parameters with SQL commands.  A database
 * is created and then data is inserted with positioned parameters and then
 * with named parameters.  Finally the data is read out and displayed.
 * 
 * The following methods are illustrated:
 *     Raima.Rdm.RdmConnection:
 *         ChangeDatabase(string) Method
 *         Close() Method
 *         CreateCommand() Method
 *         Open() Method
 *         RdmConnection(String) Constructor
 *     Raima.Rdm.RdmCommand:
 *         CommandText Property
 *         ExecuteNonQuery() Method
 *         ExecuteReader() Method
 *         Parameters Property
 *     Raima.Rdm.RdmDataReader:
 *         Item[int] Property
 *         Read() Method
 *     Raima.Rdm.RdmParameterCollection
 *         Add(RdmParameter) Method
 *         Add(String, RdmType) Method
 *         Clear() Method
 *         Item[int] Property
 *         Item[string] Property
 * 
 * They are fully document in http://docs.raima.com/rdm/.
 * 
 * For simplicity this example does not catch any thrown exceptions
 * but good programming practices would dictate that this occur in
 * a full application.
 *)

let DropDatabase (conn : RdmConnection) name =
    let cmd = conn.CreateCommand()

    (* Attempt to drop this database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. *)
    try
        cmd.CommandText <- "DROP DATABASE " + name
        cmd.ExecuteNonQuery() |> ignore
    with
        | exn -> ()

let CreateThisDatabase (conn : RdmConnection) name cmds =
    let cmd = conn.CreateCommand()

    cmd.CommandText <- "CREATE DATABASE " + name
    cmd.ExecuteNonQuery() |> ignore

    cmds |>
        Array.iter(fun c ->
            cmd.CommandText <- c
            cmd.ExecuteNonQuery() |> ignore
        )

    cmd.CommandText <- "COMMIT"
    cmd.ExecuteNonQuery() |> ignore

    cmd.CommandText <- "CLOSE " + name
    cmd.ExecuteNonQuery() |> ignore

let CreateDatabases conn =
    (* Drop any existing copies of the databases. *)
    DropDatabase conn "SQL07"

    (* Attempt to create the databases *)
    CreateThisDatabase conn "SQL07" [| "CREATE TABLE Table1 (id INTEGER PRIMARY KEY, str CHAR(30))" |]

(* Create the connection with a simple connection string *)
let conn = new RdmConnection("host=local;openmode=x")
conn.Open()

CreateDatabases conn

(* Open the database *)
conn.ChangeDatabase("SQL07")

(* Create a new command *)
let cmd = conn.CreateCommand()

(* Insert three rows into the table using positioned parameters *)
cmd.CommandText <- "INSERT INTO Table1 VALUES ?, ?"
cmd.Parameters.Add(new RdmParameter()) |> ignore
cmd.Parameters.Add(new RdmParameter()) |> ignore

[1..3] |>
    List.iter(fun ii ->
        cmd.Parameters.[0].Value <- ii
        cmd.Parameters.[1].Value <- "positioned " + (string ii)
        cmd.ExecuteNonQuery() |> ignore
    )

(* Insert three rows into the table using named parameters *)
cmd.Parameters.Clear()
cmd.CommandText <- "INSERT INTO Table1 VALUES :idparm, :strparm"
cmd.Parameters.Add("idparm", RdmType.Int32) |> ignore
cmd.Parameters.Add("strparm", RdmType.AnsiString) |> ignore

[4..6] |>
    List.iter(fun ii ->
        cmd.Parameters.["idparm"].Value <- ii
        cmd.Parameters.["strparm"].Value <- "named " + (string ii)
        cmd.ExecuteNonQuery() |> ignore
    )

(* Scan through the data, displaying the rows *)
cmd.Parameters.Clear()
cmd.CommandText <- "SELECT id, str FROM Table1"
let rdr1 = cmd.ExecuteReader()

while (rdr1.Read()) do
    printfn "%s -- %s" (string (rdr1.Item(0))) (string (rdr1.Item(1)))

conn.Close()

printfn "Hit <Enter> to continue"
System.Console.ReadLine() |> ignore

